@@ -6,12 +6,14 @@ import unicodedata
 import shout
 import daemon
 import daemon.pidfile
-from signal import SIGTSTP
+from signal import SIGTSTP, SIGABRT
 import sys, os
-import jukebox_core.api
+from jukebox.jukebox_core import api
 
 
 class Command(BaseCommand):
+    skipCurrentSong = False
+
     option_list = BaseCommand.option_list + (
         make_option(
             "--start",
@@ -25,6 +27,12 @@ class Command(BaseCommand):
             dest="stop",
             help="Stop shoutcast streaming"
         ),
+        make_option(
+            "--fg",
+            action="store_true",
+            dest="fg",
+            help="Start shoutcast streaming in foreground"
+        ),
         make_option(
             "--host",
             action="store",
@@ -50,7 +58,7 @@ class Command(BaseCommand):
             os.path.abspath(__file__)
         ) + "/../../daemon.pid"
 
-        if options["start"]:
+        if options["start"] or options["fg"]:
             if (options["host"] is None or
                 options["port"] is None or
                 options["password"] is None
@@ -95,23 +103,44 @@ class Command(BaseCommand):
                 print "Error: " + str(exception)
                 return
 
-            print "Starting jukebox_shout daemon..."
-            self.daemon = daemon.DaemonContext(
-                uid=os.getuid(),
-                gid=os.getgid(),
-                pidfile=pid,
-                working_directory=os.getcwd(),
-                detach_process=True,
-                signal_map={
-                    SIGTSTP: self.shutdown
-                }
-            )
+            if options["start"]:
+                print "Starting jukebox_shout daemon..."
+                self.daemon = daemon.DaemonContext(
+                    uid=os.getuid(),
+                    gid=os.getgid(),
+                    pidfile=pid,
+                    working_directory=os.getcwd(),
+                    detach_process=True,
+                    signal_map={
+                        SIGTSTP: self.shutdown,
+                        SIGABRT: self.skipSong
+                    }
+                )
 
-            with self.daemon:
-                self.shout.open()
-                songs_api = jukebox_core.api.songs()
-                while 1:
-                    self.sendfile(songs_api.getNextSong())
+                with self.daemon:
+                    self.shout.open()
+
+                    print "Register player"
+                    pid = int(open(pidFile).read())
+                    players_api = api.players()
+                    players_api.add(pid)
+
+                    songs_api = api.songs()
+                    while 1:
+                        self.sendfile(songs_api.getNextSong())
+
+            elif options["fg"]:
+               self.shout.open()
+
+               print "Register player"
+               pid = os.getpid()
+               players_api = api.players()
+               players_api.add(pid)
+
+               songs_api = api.songs()
+               while 1:
+                   song = songs_api.getNextSong()
+                   self.sendfile(song)
 
         elif options["stop"]:
             if not os.path.exists(pidFile):
@@ -121,6 +150,10 @@ class Command(BaseCommand):
             print "Stopping daemon..."
             pid = int(open(pidFile).read())
             os.kill(pid, SIGTSTP)
+
+            print "Unregister player " + str(pid)
+            players_api = api.players()
+            players_api.remove(pid)
         else:
             self.print_help("jukebox_shout", "help")
 
@@ -129,21 +162,35 @@ class Command(BaseCommand):
         self.daemon.close()
         sys.exit(0)
 
+    def skipSong(self, signal, action):
+        self.skipCurrentSong = True
+
     def sendfile(self, song_instance):
-        if not os.path.exists(song_instance.Filename):
-            print "File not found: %s" %  song_instance.Filename
+        if not os.path.exists(song_instance.Filename.encode('utf8')):
+            print "File not found: %s" % (
+                song_instance.Filename.encode('utf8'))
             return
 
-        print "Streaming file %s" % song_instance.Filename
-        f = open(song_instance.Filename)
+        print "Streaming file %s" % song_instance.Filename.encode('utf8')
+        f = open(song_instance.Filename.encode('utf8'))
         self.shout.set_metadata({"song": self.getMetaData(song_instance)})
         while 1:
-            print "sending..."
-            buf = f.read(4096)
-            if not len(buf):
+            if self.skipCurrentSong:
+                print "skipping current song"
+                self.shout.sync()
+                self.skipCurrentSong = False
                 break
-            self.shout.send(buf)
-            self.shout.sync()
+            else:
+                print "sending..."
+                buf = f.read(4096)
+                if not len(buf):
+                    break
+                try:
+                    self.shout.send(buf)
+                except shout.ShoutException, exc:
+                    print "Error: " + str(exc)
+                    break
+                self.shout.sync()
         f.close()
 
     def getMetaData(self, song_instance):
